searchNodes=[{"ref":"ExopPlug.html","title":"ExopPlug","type":"module","doc":"Offers a simple DSL to define a plug with number of actions and parameters with validation checks. Then you use this plug in corresponding controller and that&#39;s it: once HTTP request comes, your controller&#39;s plug takes an action: it figures out whether this particular HTTP request&#39;s parameters should be validated or not, and if yes - validates them. If parameters pass the validation ExopPlug returns Plug.Conn (as usual plug does), if not - it returns an error-tuple as described here. ExopPlug doesn&#39;t transform your HTTP request nor Plug.Conn.t() structure. So, if you define get &#39;/user/:user_id&#39; in your router you receive %{&quot;user_id&quot; =&gt; &quot;1&quot;} for the request http://localhost:4000/user/1. There is no any coercion or type inference done under the scenes. Example # your plug defmodule MyAppWeb.UserControllerPlug do use ExopPlug action(:show, params: %{&quot;id&quot; =&gt; [type: :string, length: %{min: 5}]}) end # in your controller defmodule MyAppWeb.UserController do use MyAppWeb, :controller plug MyAppWeb.UserControllerPlug # ... def show(conn, params) do json(conn, params) end # ... end action options Apart from mandatory :params option with which you define an action parameters checks, you are able to use other additional options: :on_fail - with this option you specify a callback function which is invoked if a parameter fails your validation Example: defmodule MyAppWeb.UserControllerPlug do use ExopPlug action(:show, params: %{&quot;id&quot; =&gt; [type: :string]}, on_fail: &amp;__MODULE__.on_fail/3) def on_fail(conn, action_name, errors_map) do Plug.Conn.assign(conn, :errors, errors_map) end end Exop parameter options When you&#39;re defining your action&#39;s checks you already have a number of useful checks and options which come with Exop, for example, :coerce_with may be very useful. Check Exop docs to find out more available options."},{"ref":"ExopPlug.html#action/2","title":"ExopPlug.action/2","type":"macro","doc":"Defines incoming parameters validation for your Phoenix controller&#39;s action."},{"ref":"readme.html","title":"ExopPlug","type":"extras","doc":"ExopPlug This library provides a convenient way to validate incoming parameters of your Phoenix application&#39;s controllers by offering you small but useful DSL which makes as little magic behind the scenes as possible leaves you all the control under HTTP request."},{"ref":"readme.html#table-of-contents","title":"ExopPlug - Table of Contents","type":"extras","doc":"CHANGELOG Installation How it works Step-by-step 1. create a plug 2. in a controller More examples coercing rely on action_fallback respond directly from on_fail callback LICENSE"},{"ref":"readme.html#installation","title":"ExopPlug - Installation","type":"extras","doc":"def deps do [ {:exop, &quot;~&gt; 1.4&quot;}, {:exop_plug, &quot;~&gt; 1.1&quot;} ] end"},{"ref":"readme.html#how-it-works","title":"ExopPlug - How it works","type":"extras","doc":"Basically speaking ExopPlug utilizes the power of Exop library by generating in compile time Exop-operations for your actions defined in a plug. So, you define a plug with number of actions and parameters with validation checks. Then you use this plug in corresponding controller and that&#39;s it: once HTTP request comes, your controller&#39;s plug takes an action: it figures out whether this particular HTTP request&#39;s parameters should be validated or not, and if yes - validates them. If parameters pass the validation ExopPlug returns Plug.Conn (as usual plug does), if not - it returns Exop&#39;s error-tuple as described here. And it is up to you then to decide how you want to handle the result. Please keep in mind: ExopPlug doesn&#39;t transform your HTTP request nor Plug.Conn.t() structure. So, if you define get &#39;/user/:user_id&#39; in your router you receive %{&quot;user_id&quot; =&gt; &quot;1&quot;} for the request http://localhost:4000/user/1. There is no any coercion or type inference done under the scenes. (if such feature(-s) will be requested, they&#39;ll be added and you&#39;ll need to specify them explicitly in an action definition)"},{"ref":"readme.html#step-by-step","title":"ExopPlug - Step-by-step","type":"extras","doc":"1. create a plug Create a new module plug and define actions with parameters you want to validate. A parameter&#39;s validations are the same as Exop has for an operation parameter checks. You can find all them here along with other features like coercion. defmodule MyAppWeb.UserControllerPlug do use ExopPlug action(:show, params: %{&quot;id&quot; =&gt; [type: :string, length: %{min: 5}]}, on_fail: &amp;__MODULE__.on_fail/3) def on_fail(conn, action_name, errors_map) do Plug.Conn.assign(conn, :errors, errors_map) end end Here we also defined an on_fail callback. This 3-arity function is called when an action&#39;s parameters failed the specified validation. 2. in a controller Simply add plug MyAppWeb.UserControllerPlug at the top of your controller. defmodule MyAppWeb.UserController do use MyAppWeb, :controller plug MyAppWeb.UserControllerPlug # ... def show(conn, params) do json(conn, params) end # ... end Now, if you receive invalid parameters for your show you get (for example) errors: %{&quot;id&quot; =&gt; [&quot;has wrong type&quot;]}} within your Plug.Conn assigns (as you earlier specified in your plug). And then it is up to you how to deal with this errors map."},{"ref":"readme.html#more-examples","title":"ExopPlug - More examples","type":"extras","doc":"The power of ExopPlug actually is provided by Exop and its validation capabilities. Basically, every check you use in Exop you can apply for an action&#39;s parameter in your plug. Below I put a couple of examples just in order to show different checks and things you can do with a parameter or validation errors. coercing In the example above there is one tricky thing which I&#39;ve just omit for the sake of example: with get &#39;/user/:user_id&#39; route you&#39;ll get user_id parameter as string, always, because it is path parameter. But you can coerce it before the validation: action(:show, params: %{&quot;id&quot; =&gt; [type: :integer, coerce_with: &amp;__MODULE__.coerce_integer/2]}) def coerce_integer({_, param_value}, _) when is_binary(param_value) do {integer, &quot;&quot;} = Integer.parse(param_value) integer end (and again: read more about :coerce_with option in Exop docs) rely on action_fallback After assigning errors to a connection you can later pattern-match it in a controller&#39;s action and invoke your action_fallback&#39;s fallback controller: # in a plug ... action(:show, params: %{&quot;id&quot; =&gt; [type: :integer]}, on_fail: &amp;__MODULE__.on_fail/3) def on_fail(conn, action_name, errors_map) do Plug.Conn.assign(conn, :errors, errors_map) end # in a controller ... action_fallback MyAppWeb.FallbackController def show(%Plug.Conn{assigns: %{errors: errors_map}}, _params) do {:error, errors_map} end # in the fallback controller ... def call(conn, {:error, errors_map}) do json(conn, errors_map) end respond directly from on_fail callback It might be useful not to assign validation errors to a connection, but respond immediately: action(:show, params: %{&quot;id&quot; =&gt; [type: :integer]}, on_fail: &amp;__MODULE__.on_fail/3) def on_fail(conn, action_name, errors_map) do response = %{ action: action_name, errors: errors_map } Phoenix.Controller.json(conn, response) end"},{"ref":"readme.html#license","title":"ExopPlug - LICENSE","type":"extras","doc":"Copyright © 2019 - 2020 Andrey Chernykh ( andrei.chernykh@gmail.com ) This work is free. You can redistribute it and/or modify it under the terms of the MIT License. See the LICENSE file for more details."}]